/*
 * PIC32 peripheral configuration and initialisation.
 *
 * Copyright (C) 2017, 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __INIT_H__
#define __INIT_H__

#include "pic32_c.h"



/* Basic initialisation. */

void init_memory(void);
void init_pins(void);
void init_outputs(void);



/* Peripheral pin configuration. */

void lock_config(void);
void unlock_config(void);



/* Convenience operations. */

void interrupts_on(void);



/* DMA configuration. */

void init_dma(void);

void dma_init(int channel, uint8_t pri);

void dma_init_interrupt(int channel, uint8_t conditions,
                        uint8_t pri, uint8_t sub);

void dma_off(int channel);

void dma_on(int channel);

void dma_set_auto_enable(int channel, int enable);

void dma_set_chaining(int channel, enum dma_chain chain);

void dma_set_enable(int channel, int enable);

void dma_set_interrupt(int channel, uint8_t int_num, int enable);

void dma_set_interrupt_enable(int channel, int enable);

void dma_set_receive_events(int channel, int enable);

void dma_set_cell(int channel, uint32_t size);
void dma_set_destination(int channel, uint32_t start_address, uint32_t size);
void dma_set_source(int channel, uint32_t start_address, uint32_t size);

void dma_set_transfer(int channel,
                  uint32_t source_start_address, uint32_t source_size,
                  uint32_t destination_start_address, uint32_t destination_size,
                  uint32_t cell_size);

int DMA_INT_FLAGS(int channel, uint8_t flags);

uint32_t DMA_IPC_PRI(int channel, uint8_t pri, uint8_t sub);



/* External interrupt configuration. */

void int_init_interrupt(int int_num, uint8_t pri, uint8_t sub);

int INT_INT_FLAGS(int int_num, uint8_t flags);

uint32_t INT_IPC_PRI(int int_num, uint8_t pri, uint8_t sub);
uint32_t INT_IPC_REG(int int_num);



/* Output compare configuration. */

void oc_init(int unit, uint8_t mode, int timer);

void oc_init_interrupt(int unit, uint8_t pri, uint8_t sub);

void oc_on(int unit);

void oc_set_pulse(int unit, uint32_t start);

void oc_set_pulse_end(int unit, uint32_t end);

int OC_INT_FLAGS(int unit, uint8_t flags);

uint32_t OC_IPC_PRI(int unit, uint8_t pri, uint8_t sub);
uint32_t OC_IPC_REG(int unit);



/* Parallel mode configuration. */

void init_pm(void);

void pm_init(int port, uint8_t mode);

void pm_init_interrupt(int port, uint8_t pri, uint8_t sub);

void pm_off(int port);

void pm_on(int port);

void pm_set_output(int port, int write_enable, int read_enable);

int PM_INT_FLAGS(int port, uint8_t flags);

uint32_t PM_IPC_PRI(int port, uint8_t pri, uint8_t sub);
uint32_t PM_IPC_REG(int port);



/* Timer configuration. */

void timer_init(int timer, uint8_t prescale, uint16_t limit);

void timer_init_interrupt(int timer, uint8_t pri, uint8_t sub);

int timer_interrupt_number(int timer);

void timer_on(int timer);

int TIMER_INT_FLAGS(int timer, uint8_t flags);

uint32_t TIMER_IPC_PRI(int timer, uint8_t pri, uint8_t sub);
uint32_t TIMER_IPC_REG(int timer);



/* UART configuration. */

void uart_init(int uart, int fpb, uint32_t baudrate);

void uart_init_interrupt(int uart, uint8_t conditions,
                         uint8_t pri, uint8_t sub);

void uart_on(int uart);

int UART_INT_FLAGS(int uart, uint8_t flags);

uint32_t UART_IPC_PRI(int uart, uint8_t pri, uint8_t sub);
uint32_t UART_IPC_REG(int uart);

#endif /* __INIT_H__ */
